<?php
/**
 * Created by PhpStorm.
 * User: pub
 * Date: 2019-08-16
 * Time: 11:55
 */

namespace Drupal\document_version;



use DateTimeZone;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\document_version\Entity\DocumentEntity;
use Drupal\document_version\Entity\DocumentEntityInterface;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;


/**
 * Implements a service for handling entities
 *
 * @ingroup document_version
 */
class DocumentManager implements DocumentManagerInterface {
  
  
  /**
   * gets the active document
   *
   * @param \Drupal\document_version\Entity\DocumentEntityInterface $entity
   *
   * @return \Drupal\document_version\Entity\DocumentEntityInterface
   */
  public function getActive(DocumentEntityInterface $entity) {
    $query = $this->buildPastDocumentsQuery($entity);
    $query->pager(1);
    $ids = $query->execute();
    return DocumentEntity::load(array_pop($ids));
  }
  
  /**
   * gets documents dated in the future
   *
   * @param \Drupal\document_version\Entity\DocumentEntityInterface $entity
   *
   * @return \Drupal\document_version\Entity\[]
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getFutureDocuments(DocumentEntityInterface $entity) {
    $timezone = new DateTimeZone(date_default_timezone_get());
    $current_time = new DrupalDateTime('now', $timezone);
    $query = $this->buildBaseQuery($entity);
    $query->condition('effective_date', $current_time->getTimestamp(), '<=');
    $query->sort('effective_date', 'DESC');
    $ids = $query->execute();
    
    return $this->loadEntities($ids);
  }
  
  /**
   * gets documents in the past
   *
   * @param \Drupal\document_version\Entity\DocumentEntityInterface $entity
   *
   * @return []|\Drupal\document_version\Entity\[]
   */
  public function getPastDocuments(DocumentEntityInterface $entity) {
    $query = $this->buildPastDocumentsQuery($entity);
    $ids = $query->execute();
    try {
      return $this->loadEntities($ids);
    } catch (\Exception $e) {
      \Drupal::logger('document_version')->error($e->getMessage());
    }
    return [];
  }
  
  /**
   * builds query to get active and past documents
   *
   * @param \Drupal\document_version\Entity\DocumentEntityInterface $entity
   *
   * @return \Drupal\Core\Entity\Query\QueryInterface
   */
  private function buildPastDocumentsQuery($entity) {
    $timezone = new DateTimeZone('UTC');
    $current_time = new DrupalDateTime('now', $timezone);
    $query = $this->buildBaseQuery($entity);
    $query->condition('effective_date', $current_time->getTimestamp(), '<=');
    $query->sort('effective_date', 'DESC');
    return $query;
  }
  
  /**
   * @param \Drupal\document_version\Entity\DocumentEntityInterface $entity
   *
   * @return \Drupal\document_version\Entity\[]
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getAllDocuments(DocumentEntityInterface $entity) {
    $query = $this->buildBaseQuery($entity);
    $query = $query->sort('effective_date', 'DESC');
    $ids = $query->execute();
    
    return $this->loadEntities($ids);

  }
  
  /**
   * @param \Drupal\document_version\Entity\DocumentEntityInterface $entity
   *
   * @return bool
   */
  public function isActive(DocumentEntityInterface $entity) {
    return ($this->getActive($entity) === $entity);
  }
  
  /**
   * generates base query for all service manager queries
   * @param \Drupal\document_version\Entity\DocumentEntityInterface $entity
   *
   * @return \Drupal\Core\Entity\Query\QueryInterface
   */
  private function buildBaseQuery(DocumentEntityInterface $entity) {
    
    $query = \Drupal::entityQuery('document_entity');
    $query->condition('status', 1);
    $query->condition('type', $entity->bundle());
    $query->condition('document_type', $entity->getDocumentType()->id(), '=');
    
    return $query;
  }
  
  /**
   * @param array $ids
   *
   * @return \Drupal\document_version\Entity\[]
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function loadEntities ($ids) {
    /**
     * @var \Drupal\document_version\Entity\[] $entities
     */
    $entities =  \Drupal::entityTypeManager()
      ->getStorage('document_version')
      ->loadMultiple($ids);
  
    return $entities;
  }
  
  /**
   * gets the active document given a document_group_tid and document_type_tid
   * @param string $bundle
   * @param int $document_type_tid
   *
   * @return null|\Drupal\document_version\Entity\DocumentEntity
   */
  public function loadFromGroups($bundle, $document_type_tid) {
    
    $timezone = new DateTimeZone('UTC');
    $current_time = new DrupalDateTime('now', $timezone);
    
    \Drupal::logger('DocumentManager')->notice('UTC timezone is: @timezone and time is @time',
      ['@timezone' => $timezone, '@time' => $current_time->getTimestamp()]);
    $query = \Drupal::entityQuery('document_entity');
    $query->condition('status', 1);
    $query->condition('type', $bundle);
    
    $query->condition('document_type', $document_type_tid, '=');
    $query->condition('effective_date', $current_time->getTimestamp(), '<=');
    $query->sort('effective_date', 'DESC');
    $query->pager(1);
    
    $ids = $query->execute();
    $id = array_pop($ids);
    if ($id !== null) {
      return DocumentEntity::load($id);
    } else {
      return null;
    }
  }
  
  /**
   * @return string
   * @param string $str
   *
   * @return string|void
   */
 public function slug($str) {
   return strtolower(trim(preg_replace('~[^0-9a-z]+~i',
       '-',
       html_entity_decode(preg_replace('~&([a-z]{1,2})(?:acute|cedil|circ|grave|lig|orn|ring|slash|th|tilde|uml);~i',
         '$1',
         htmlentities($str, ENT_QUOTES, 'UTF-8')),
         ENT_QUOTES, 'UTF-8')),
       '-')
   );
 }
  
  /**
   * @param $entity
   *
   * @return array|mixed|string|null
   */
 public function getDownloadLink($entity) {
   if (get_class($entity ) === 'Drupal\document_version\Entity\DocumentEntity') {
     /**
      * @var DocumentEntity $entity
      */
     $config = \Drupal::config('document_version.settings');
     $link = '/'. $config->get('document_base_path');
     /**
      * @var \Drupal\taxonomy\Plugin\views\wizard\TaxonomyTerm $document_type
      */
     $document_type = $entity->getDocumentType();
     
     $link .= '/' . $entity->bundle() . '/' . $document_type->get('field_simple_document_url')->value;
     
     return $link;
   }

 }
}
