<?php

namespace Drupal\document_version\Entity;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityPublishedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\user\UserInterface;
use Drupal\Core\Datetime\DrupalDateTime;
/**
 * Defines the Document entity entity.
 *
 * @ingroup document_version
 *
 * @ContentEntityType(
 *   id = "document_entity",
 *   label = @Translation("Document entity"),
 *   bundle_label = @Translation("Document entity type"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\document_version\DocumentEntityListBuilder",
 *     "views_data" = "Drupal\document_version\Entity\DocumentEntityViewsData",
 *
 *     "form" = {
 *       "default" = "Drupal\document_version\Form\DocumentEntityForm",
 *       "add" = "Drupal\document_version\Form\DocumentEntityForm",
 *       "edit" = "Drupal\document_version\Form\DocumentEntityForm",
 *       "delete" = "Drupal\document_version\Form\DocumentEntityDeleteForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\document_version\DocumentEntityHtmlRouteProvider",
 *     },
 *     "access" = "Drupal\document_version\DocumentEntityAccessControlHandler",
 *   },
 *   base_table = "document_entity",
 *   translatable = FALSE,
 *   admin_permission = "administer document entity entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "bundle" = "type",
 *     "uuid" = "uuid",
 *     "langcode" = "langcode",
 *     "published" = "status",
 *   },
 *   links = {
 *     "canonical" = "/admin/structure/document_entity/{document_entity}",
 *     "add-page" = "/admin/structure/document_entity/add",
 *     "add-form" = "/admin/structure/document_entity/add/{document_entity_type}",
 *     "edit-form" = "/admin/structure/document_entity/{document_entity}/edit",
 *     "delete-form" = "/admin/structure/document_entity/{document_entity}/delete",
 *     "collection" = "/admin/structure/document_entity",
 *   },
 *   bundle_entity_type = "document_entity_type",
 *   field_ui_base_route = "entity.document_entity_type.edit_form"
 * )
 */
class DocumentEntity extends ContentEntityBase implements DocumentEntityInterface {

  use EntityChangedTrait;
  use EntityPublishedTrait;

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    $values += [
      'user_id' => \Drupal::currentUser()->id(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    // Add the published field.
    $fields += static::publishedBaseFieldDefinitions($entity_type);
  
    $fields['document_type'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Document URL'))
      ->setDescription(t('The URL of the Document'))
      ->setSetting('target_type', 'taxonomy_term')
      ->setSetting('handler', 'default')
      ->setSetting('handler_settings', [
        'target_bundles' => ['standard_document_type'],
      ])
      ->setDisplayOptions('view', [
        'type' => 'entity_reference_label',
        'label' => 'hidden',
        'weight' => -6,
      ])
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => -6,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);
  
    $fields['file'] = BaseFieldDefinition::create('file')
      ->setLabel(t('Document Upload'))
      ->setDescription(t('The file to be served'))
      ->setSetting('max_filesize', '8M')
      ->setSetting('file_extensions', 'pdf docx doc txt')
      ->setSetting('file_directory', 'documents/[date:custom:Y]/[date:custom:M]')
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'file_url_plain',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'file_generic',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);
  

  
    $fields['effective_date'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Effective Date'))
      ->setDescription(t('The date the document becomes active.'))
      ->setDefaultValue(time())
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'weight' => -5,
        'type' => 'datetime_timestamp',
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);
    
    $fields['status']->setDescription(t('A boolean indicating whether the Document entity is published.'))
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'weight' => -3,
      ]);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the entity was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the entity was last edited.'));

    return $fields;
  }
  
  /**
   * Get the document type
   *
   * @return \Drupal\taxonomy\Entity\Term
   */
  public function getDocumentType() {
    return $this->get('document_type')->entity;
  }
  
  /**
   *
   * @return \Drupal\simple_document\Entity\SimpleDocumentInterface
   *   The called Simple Document entity.
   * /
   * public function setDocumentType($tid);
   *
   * /**
   * @return mixed
   */
  public function getEffectiveDate() {
    return $this->get('effective_date')->value;
  }
  
  /**
   * @param int $timestamp
   *
   * @return \Drupal\document_version\Entity\DocumentEntityInterface
   *   The called Simple Document entity.
   */
  public function setEffectiveDate($timestamp) {

    $effective_time = DrupalDateTime::createFromTimestamp($timestamp, \Drupal\Core\Session\drupal_get_user_timezone());
    $this->set('effective_date', $effective_time->getTimestamp());
    return $this;
  }
  
  /**
   * @param int $tid
   *
   * @return \Drupal\document_version\Entity\DocumentEntityInterface
   *   The called Simple Document entity.
   */
  public function setDocumentType($tid) {
    $this->set('document_type', $tid);
    return $this;
  }
  
  /**
   * returns timestamp of effective date
   * @return false|int
   */
  public function getEffectiveDateTimestamp () {
    return strtotime($this->getEffectiveDate());
  }
  
  public function save() {
    $entity = parent::save();
    
    return $entity;
  }
  
  
}
